# plugs/infoitem.py
#
#

""" information items .. keyword/description pairs """

__copyright__ = 'this file is in the public domain'

from gozerbot.tests import tests
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.redispatcher import rebefore, reafter
from gozerbot.ignore import shouldignore
from gozerbot.datadir import datadir
from gozerbot.persist.persist import Persist
from gozerbot.generic import lockdec, cchar, rlog, handle_exception
from gozerbot.aliases import aliases
from gozerbot.plughelp import plughelp
from gozerbot.callbacks import callbacks
from gozerbot.users import users
from gozerbot.config import config
from gozerbot.database.alchemy import Base, create_session, trans, create_all
from datetime import datetime
from time import localtime
from sqlalchemy import Column, String, Integer, Text, DateTime, ForeignKey, Sequence
import sqlalchemy as sa

class InfoItems(Base):
    __tablename__ = 'infoitems'
    __table_args__ = {'useexisting': True}
    indx = Column('indx', Integer, Sequence('infoitems_indx_seq', optional=True), primary_key=True)
    item = Column('item', String(255), nullable=False)
    description = Column('description', Text, nullable=False)
    userhost = Column('userhost', String(255), ForeignKey('userhosts.userhost'), nullable=False)
    time = Column('time', DateTime, nullable=False)

    def __init__(self, item, description, userhost, ttime=None):
        self.time = ttime and ttime or datetime.now()
        self.item = item.lower()
        self.description = description
        self.userhost = userhost

## BEGIN UPGRADE PART

def upgrade():
    rlog(10, 'infoitem', 'upgrading')
    teller = 0
    dbdir = datadir + os.sep + 'db' + os.sep + 'infoitem.db'
    if os.path.exists(dbdir):
        try:
            from gozerbot.database.db import Db
            db = Db(dbtype='sqlite')
            db.connect('db/infoitem.db')
            result = db.execute(""" SELECT * FROM infoitems """)
            if result:
                for i in result:
                    info.add(*i[1:])
                    teller += 1
        except Exception, ex:
            handle_exception()
    else:
        oldfile = datadir + os.sep + 'old' + os.sep + 'infoitems'
        try:
            from gozerbot.utils.generic import dosed
            from gozerbot.compat.persist import Persist
            oldinfo = Persist(oldfile)
            assert(oldinfo)
            if not oldinfo.data:
                return
            for item, descrlist in oldinfo.data.iteritems():
                for descr in descrlist:
                    try:
                        info.add(item, descr, 'none', 0)
                    except Exception, ex:
                        handle_exception()
                    teller += 1
        except IOError, ex:
            if "No such file" in str(ex):
                rlog(10, 'infoitem', 'nothing to upgrade')
        except Exception, ex:
            rlog(10, 'infoitem', "can't upgrade: %s" % str(ex))
            return
    rlog(10, 'infoitem', 'upgraded %s infoitems' % str(teller))
    return teller

import thread, os, time

plughelp.add('infoitem', 'also known as factoids .. info can be retrieved \
by keyword or searched')

infolock = thread.allocate_lock()

# create lock descriptor
locked = lockdec(infolock)

class InfoitemsDb(object):

    """ information items """

    @trans 
    def add(self, session, item, description, userhost, ttime, channel="", name=""):
        """ add an item """
        item = item.lower()
        result = 0
        try:
            newitem = InfoItems(item, description, userhost, datetime.fromtimestamp(ttime))
            session.add(newitem)
            result = 1
        except Exception, ex:
            rlog(10, 'infoitem', "can't add item: %s" % str(ex))
            raise ex
        return result

    def get(self, item):
        """ get infoitems """
        s = create_session()
        item = item.lower()
        result = [r.description for r in s.query(InfoItems).filter_by(item=item)]
        return result

    @trans
    def delete(self, session, indexnr):
        """ delete item with indexnr  """
        item = session.query(InfoItems).filter_by(indx=indexnr).first()
        try:
            session.delete(item)
            result = 1
        except Exception, ex:
            rlog(10, 'infoitem', "can't delete item %s: %s" % (str(indexnr), str(ex)))
            raise ex
        return result

    @trans
    def deltxt(self, session, item, txt):
        """ delete item with matching txt """
        items = session.query(InfoItems).filter(InfoItems.item==item.lower
                        ).filter(InfoItems.description.like('%%%s%%' % txt)).all()
        result = 0
        try:
            for n, i in enumerate(items):
                session.delete(i)
                result = n + 1
        except Exception, ex:
            rlog(10, 'infoitem', "can't delete items like %s: %s" % (str(txt), str(ex)))
            raise ex
        return result

    def size(self):
        """ return number of items """
        s = create_session()
        count = s.query(sa.func.count(InfoItems.indx)).first()[0]
        return count

    def searchitem(self, search):
        """ search items """
        s = create_session() 
        result = s.query(InfoItems).filter(InfoItems.item.like('%%%s%%' % search)).all()
        return result

    def searchdescr(self, search):
        """ search descriptions """
        s = create_session()
        result = s.query(InfoItems).filter(InfoItems.description.like('%%%s%%' % search)).all()
        return result


info = InfoitemsDb()
create_all('infoitem')
assert(info)

def size():
    """ return number of infoitems """
    return info.size()

def search(what, queue):
    rlog(10, 'infoitem', 'searched for %s' % what)
    result = info.searchitem(what)   
    if not result:
        return
    res = []
    for i in result:
        queue.put_nowait(i.description)
    result = info.searchdescr(what)   
    if not result:
        return
    for i in result:
        queue.put_nowait("[%s] %s" % (i.item, i.description))

def infopre(bot, ievent):
    """ see if info callback needs to be called """
    cc = cchar(bot, ievent)
    if ievent.origtxt and  ievent.origtxt[0] in cc and not ievent.usercmnd \
and ievent.txt:
        return 1

def infocb(bot, ievent):
    """ implement a !infoitem callback """
    if not shouldignore(ievent.userhost):
        if 'handle_question' in bot.state['allowed'] or users.allowed(ievent.userhost, 'USER'):
             data = info.get(ievent.txt)
             if data:
                ievent.reply('%s is ' % ievent.txt, data , dot=True)

callbacks.add('PRIVMSG', infocb, infopre)

def handle_infoupgrade(bot, ievent):
    if info.size():
         if '-f' in ievent.optionset:
             pass
         else:
             ievent.reply('there are already infoitems in the main database .. not upgrading')
             ievent.reply('use the -f option to force an upgrade')
             return
    ievent.reply('upgrading infoitems')
    nritems = upgrade()
    ievent.reply('%s items upgraded' % nritems)

cmnds.add('info-upgrade', handle_infoupgrade, 'OPER', options={'-f': ''})

def handle_infosize(bot, ievent):
    """ info-size .. show number of information items """
    ievent.reply("we have %s infoitems" % info.size())

cmnds.add('info-size', handle_infosize, ['USER', 'WEB', 'CLOUD'])
examples.add('info-size', 'show number of infoitems', 'info-size')
tests.add('info-size', 'we have (\d+) infoitems')

def handle_addinfoitem(bot, ievent):
    """ <keyword> = <description> .. add information item """
    try:
        (what, description) = ievent.groups
    except ValueError:
        ievent.reply('i need <item> <description>')
        return
    if len(description) < 3:
        ievent.reply('i need at least 3 chars for the description')
        return
    what = what.strip()
    ret = info.add(what, description, ievent.userhost, time.time())
    if ret:
        ievent.reply('item added')
    else:
        ievent.reply('unable to add item')

rebefore.add(10, '^(.+?)\s+=\s+(.+)$', handle_addinfoitem, ['USER', \
'INFOADD'], allowqueue=False)
examples.add('=', 'add description to item', 'dunk = top')
tests.add('gozerbot = top bot', 'item added')

def handle_question(bot, ievent):
    """ <keyword>? .. ask for information item description """
    try:
        what = ievent.groups[0]
    except TypeError:
        what = ievent.rest
    except IndexError:
        ievent.reply('i need a argument')
        return
    what = what.strip().lower()
    infoitems = info.get(what)
    if infoitems:
        ievent.reply("%s is " % what, infoitems, dot=True)
    else:
        ievent.reply('nothing known about %s' % what)

reafter.add(10, '^(.+)\?$', handle_question, ['USER', 'WEB', 'JCOLL', \
'CLOUD'], allowqueue=True)
reafter.add(10, '^\?(.+)$', handle_question, ['USER', 'WEB', 'JCOLL', \
'CLOUD'], allowqueue=True)
cmnds.add('?', handle_question, ['USER', 'INFOADD'], allowqueue=False)
examples.add('?', 'show infoitems of <what>', '1) test? 2) ?test')
tests.add('gozerbot?', 'top bot')
tests.add('?gozerbot', 'top bot')

def handle_forget(bot, ievent):
    """ forget <keyword> <txttomatch> .. remove information item where \
        description matches txt given """
    if len(ievent.args) > 1:
        what = ' '.join(ievent.args[:-1])
        txt = ievent.args[-1]
    else:
        ievent.missing('<item> <txttomatch> (min 3 chars)')
        return
    if len(txt) < 3:
        ievent.reply('i need txt with at least 3 characters')
        return
    what = what.strip().lower()
    try:
        nrtimes = info.deltxt(what, txt)
    except KeyError:
        ievent.reply('no records matching %s found' % what)
        return
    if nrtimes:
        ievent.reply('item deleted')
    else:
        ievent.reply('delete %s of %s failed' % (txt, what))

cmnds.add('info-forget', handle_forget, ['FORGET', 'OPER'])
examples.add('info-forget', 'forget <item> containing <txt>', 'info-forget \
dunk bla')
aliases.data['forget'] = 'info-forget'
tests.add('gozerbot = top bot').add('info-forget gozerbot top', 'item deleted')

def handle_searchdescr(bot, ievent):
    """ info-sd <txttosearchfor> .. search information items descriptions """
    if not ievent.rest:
        ievent.missing('<txt>')
        return
    else:
        what = ievent.rest
    what = what.strip().lower()
    result = info.searchdescr(what)
    if result: 
        res = []
        for i in result:
            res.append("[%s] %s" % (i.item, i.description))
        ievent.reply("the following matches %s: " % what, res, dot=True)
    else:
        ievent.reply('none found')

cmnds.add('info-sd', handle_searchdescr, ['USER', 'WEB', 'CLOUD'])
examples.add('info-sd', 'info-sd <txt> ..  search description of \
infoitems', 'info-sd http')
aliases.data['sd'] = 'info-sd'
aliases.data['sl'] = 'info-sd'
tests.add('gozerbot = top bot').add('info-sd top', 'top bot')

def handle_searchitem(bot, ievent):
    """ info-si <txt> .. search information keywords """
    if not ievent.rest:
        ievent.missing('<txt>')
        return
    else:
        what = ievent.rest
    what = what.strip().lower()
    result = info.searchitem(what)
    if result:
        res = []
        for i in result:
            res.append("[%s] %s" % (i.item, i.description))
        ievent.reply("the following matches %s: " % what, res, dot=True)
    else:
        ievent.reply('none found')

cmnds.add('info-si', handle_searchitem, ['USER', 'WEB', 'CLOUD'])
examples.add('info-si', 'info-si <txt> ..  search the infoitems keys', \
'info-si test')
aliases.data['si'] = 'info-si'
tests.add('gozerbot = top bot').add('info-si gozer', 'gozerbot')



# plugs/log.py
#
#

""" logging """

__copyright__ = 'this file is in the public domain'
__gendocfirst__ = ['log-on', ]
__gendoclast__ = ['log-off', ]
__depending__ = ['mail' , 'markov']

from gozerbot.generic import rlog, handle_exception, elapsedstring, dmy, \
hourmin, lockdec, strtotime
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.callbacks import callbacks, jcallbacks
from gozerbot.plughelp import plughelp
from gozerbot.irc.monitor import saymonitor
from gozerbot.xmpp.monitor import xmppmonitor
from gozerbot.aliases import aliases, aliasdel
from gozerbot.users import users
import glob, re, thread, pickle, os, mmap, time

plughelp.add('log', 'logs related commands')

# check if logs dir exists if not create it
if not os.path.isdir('logs'):
    os.mkdir('logs')

loglock = thread.allocate_lock()
locked = lockdec(loglock)

class Logs(object):

    """ hold handles to log files (per channel) """

    def __init__(self, logdir):
        self.logdir = logdir
        self.maps = {}
        self.files = {}
        self.filenames = {}
        self.lock = thread.allocate_lock()
        self.loglist = []
        try:
            loglistfile = open(self.logdir + os.sep + 'loglist', 'r')
            self.loglist = pickle.load(loglistfile)
        except:
            self.loglist = []
        rlog(0, 'logs', 'loglist is %s' % str(self.loglist))
        # open log files in append mode
        for i in glob.glob(self.logdir + os.sep + '*.log'):
            reresult = re.search('logs%s(.+)\.' % os.sep, i)
            filename = reresult.group(1)
            if not filename:
                rlog(10, 'logs', "can't determine channel name")
                continue
            logfile = open(i, 'a')
            if logfile:
                self.files[filename] = logfile
                self.filenames[filename.lower()] = i
                rlog(1, 'logs', 'adding file %s' % i)
            else:
                rlog(10, 'logs', 'failed to open %s' % i)

    def save(self):
        """ save loglist """
        try:
            self.lock.acquire()
            picklefile = open(self.logdir + os.sep + 'loglist', 'w')
            pickle.dump(self.loglist, picklefile)
            rlog(0, 'logs', 'loglist saved')
        finally:
            self.lock.release()

    def close(self):
        """ close log files """
        try:
            self.lock.acquire()
            for i in self.files.keys():
                rlog(10, 'logs', 'closing ' + str(i))
                self.files[i].close()
        finally:
            self.lock.release()
        self.save()

    def size(self, channel):
        """ get size of log file """
        if channel[0] in ['#', '!', '&', '+']:
            chan = channel[1:].lower()
        else:
            chan = channel.lower()
        try:
            size = os.stat(self.filenames[chan])[6]
            return size
        except (IOError, KeyError):
            return 0

    def add(self, channel):
        """ add logfile for channel """
        if channel[0] in ['#', '!', '&', '+']:
            chan = channel[1:].lower()
        else:
            chan = channel.lower()
        filename = self.logdir + os.sep + chan + '.log'
        rlog(10, 'logs', 'adding logfile %s' % chan)
        if self.files.has_key(chan):
            rlog(10, 'logs', 'already opened file %s' % chan)
            return 1
        try:
            logfile = open(filename,'a')
            self.files[chan] = logfile
            self.filenames[chan] = filename
            if channel not in self.loglist:
                self.loglist.append(channel)
            return 1
        except Exception, ex:
            handle_exception()
            return 0

    def logbot(self, name, ttime, channel, txt):
        """ log stuff spoken by the bot """
        if channel not in self.loglist:
            rlog(1, 'logs', "%s not in loglist" % channel)
            return
        if channel[0] in ['#', '!', '&', '+']:
            chan = channel[1:].lower()
        else:
            chan = channel.lower()
        filename = self.logdir + os.sep + chan + '.log'
        if not self.files.has_key(chan):
            if not self.add(channel):
                rlog(1, 'logs', "can't create logfile %s" % filename)
                return
        logfile = self.files[chan]
        logitem = (name, ttime, 'bot', 'bot@bot', txt)
        rlog(-1, 'logs', 'logging [%s] <%s> %s ==> %s' % (name, 'bot', txt, \
filename))
        # write the data to file (comma seperated)
        try:
            self.lock.acquire()
            logfile.write('%s,%s,%s,%s,%s\n' % logitem)
            logfile.flush()
        finally:
            self.lock.release()

    def log(self, name, ttime, ievent):
        """ log ircevent """
        if ievent.nick == ievent.bot.nick:
            ievent.nick = 'bot'
            ievent.userhost = 'bot@bot'
        channel = ievent.channel
        if channel not in self.loglist:
            rlog(1, 'logs', "%s not in loglist" % channel)
            return
        if channel[0] in ['#', '!', '&', '+']:
            chan = channel[1:].lower()
        else:
            chan = channel.lower()
        filename = self.logdir + os.sep + chan + '.log'
        if not self.files.has_key(chan):
            if not self.add(ievent.channel):
                rlog(1, 'logs', "can't create logfile %s" % filename)
                return
        # put ievent.cmnd as first txt
        if ievent.cmnd == 'PRIVMSG' or ievent.cmnd == 'Message':
            if ievent.usercmnd:
                tmp = "CMND: " + ievent.txt
            else:
                if ievent.txt.startswith('\001ACTION'):
                    txt = ievent.txt[7:-1]
                    tmp = "PRIVMSG: /me " + txt
                else:
                    tmp = "PRIVMSG: " + ievent.txt
        elif ievent.cmnd == 'MODE':
            tmp = "MODE: " + ievent.postfix
        elif ievent.cmnd == 'PART':
            tmp = "PART: " + ievent.channel
        elif ievent.cmnd == 'JOIN':
            tmp = "JOIN: " + ievent.channel
        else:
            return
        logfile = self.files[chan]
        logitem = (name, ttime, ievent.nick, ievent.userhost, tmp)
        rlog(-1, 'logs', 'logging [%s] <%s> %s ==> %s' % (name, ievent.nick, \
ievent.txt, filename))
        # write the data to file (comma seperated)
        try:
            self.lock.acquire()
            logfile.write('%s,%s,%s,%s,%s\n' % logitem)
            logfile.flush()
        finally:
            self.lock.release()

    def getmmap(self, channel):
        """ get an mmap of a channel .. as of 0.8 this just returns the 
            open logfile
        """
        if not channel:
            return
        if channel[0] in ['#', '!', '&', '+']:
            chan = channel[1:].lower()
        else:
            chan = channel.lower()
        try:
            filename = self.filenames[chan]
        except KeyError:
            rlog(10, 'log', 'logging is not enabled in %s' % channel)
            return
        logfile = open(filename, 'r')
        #size = self.size(channel)
        #if size == 0:  
        #    return None
        #logmap = mmap.mmap(logfile.fileno(), size, mmap.MAP_SHARED, \
#access=mmap.ACCESS_READ)
        return logfile

    def loop(self, logmap, nrtimes, func, withbot=False, withcmnd=False):
        """ loop over the logmap """
        logmap.seek(0)
        teller = 0
        times = 0
        for line in logmap:
            teller += 1
            if teller % 100000 == 0:
                time.sleep(0.0001)
            try:
                (botname, ttime, nick, userhost, txt) = \
line.strip().split(',', 4)
            except ValueError:
                continue
            if not withbot and userhost == 'bot@bot':
                continue
            if not withcmnd and txt.startswith('CMND:'):
                continue
            try:
                res = func(botname, ttime, nick, userhost, txt)
            except ValueError:
                continue
            if res:
                times += 1
                if nrtimes and times > nrtimes:
                    break
        logmap.close()

    def search(self, channel, what, nrtimes):
        """ search through logfile """
        res = []
        if not channel:
            return res
        andre = re.compile(' and ', re.I)
        ands = re.split(andre, what)
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return res
        res = []
        def dofunc(botname, ttime, nick, userhost, txt):
            """ search log line for matching txt """
            for j in ands:
                if txt.find(j.strip()) == -1:
                    return
            res.append((botname, ttime, nick, userhost, txt))
            return 1
        self.loop(logmap, nrtimes, dofunc)        
        if res:
            return res[:nrtimes]

    def seen(self, channel, who):
        """ get last line of who """
        result = []
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        who = who.lower()
        if '*' in who:
            who = who.replace('*', '.*')
            rewho = re.compile(who)
            def dofunc(botname, ttime, nick, userhost, txt):
                """ add line that matches mask """
                if re.match(rewho, nick.lower()):
                    result.append((botname, ttime, nick, userhost, txt))
        else:
            def dofunc(botname, ttime, nick, userhost, txt):
                """ add line that matches nick """
                if nick.lower() == who:
                    result.append((botname, ttime, nick, userhost, txt))
        self.loop(logmap, None, dofunc)
        if result:
            return result[-1]

    def back(self, channel, what, nrtimes):
        """ search through the logs backwards """
        res = []
        if not channel:
            return res
        andre = re.compile(' and ', re.I)
        ands = re.split(andre, what)
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return res
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line that matches """
            for j in ands:
                if txt.find(j.strip()) == -1:
                    return
            res.append((botname, ttime, nick, userhost, txt))
            return 1
        self.loop(logmap, None, dofunc)
        result = []
        if res:
            if len(res) > nrtimes:
                result = res[len(res)-nrtimes:]
            else:
                result = res
        result.reverse()
        return result

    def bback(self, channel, what, nrtimes):
        """ search through the bot logs backwards """
        res = []
        if not channel:
            return res
        andre = re.compile(' and ', re.I)
        ands = re.split(andre, what)
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return res
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line that matches """
            if nick != 'bot':
                return
            for j in ands:
                if txt.find(j.strip()) == -1:
                    return
            res.append((botname, ttime, nick, userhost, txt))
            return 1
        self.loop(logmap, None, dofunc, withbot=True)
        result = []
        if res:
            if len(res) > nrtimes:
                result = res[len(res)-nrtimes:]
            else:
                result = res
        result.reverse()
        return result

    def fromtime(self, channel, ftime):
        """ returns log items from a certain time """
        res = []
        if not channel:
            return res
        logmap = self.getmmap(channel)
        if not logmap:
            return res
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line that is older then given time """
            logtime = float(ttime)
            if logtime > ftime:
                res.append((botname, ttime, nick, userhost, txt))
        self.loop(logmap, None, dofunc)
        return res

    def fromtimewithbot(self, channel, ftime):
        """ returns log items from a certain time """
        res = []
        if not channel:
            return res
        logmap = self.getmmap(channel)
        if not logmap:
            return res
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line that is older then given time including bot outout """
            logtime = float(ttime)
            if logtime > ftime:
                res.append((botname, ttime, nick, userhost, txt))
        self.loop(logmap, None, dofunc, withbot=True)
        return res

    def linesback(self, channel, nrtimes):
        """ return nr log lines back """
        result = []
        if not channel:
            return result
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line """
            result.append((botname, ttime, nick, userhost, txt))
            return 1
        self.loop(logmap, None, dofunc)
        if len(result) > nrtimes:
            return result[len(result)-nrtimes:]
        else:
            return result

    def linesbacknick(self, channel, who, nrtimes):
        """ return log lines for nick """
        result = []
        if not channel:
            return result
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line said by nick """
            if nick.lower() == who.lower():
                result.insert(0, (botname, ttime, nick, userhost, txt))
                return 1
        self.loop(logmap, None, dofunc)
        return result[:nrtimes]

    def linesbacknicksearch(self, channel, who, searchitem, nrtimes):
        """ search logs for lines said by who """
        result = []
        if not channel:
            return result
        who = who.lower()
        andre = re.compile(' and ', re.I)
        ands = re.split(andre, searchitem)
        nrtimes = int(nrtimes)
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add line that is said by nick containing txt """
            if nick.lower() == who.lower():
                got = 0
                for j in ands:
                    if txt.find(j.strip()) == -1:
                        got = 0
                        break
                    got = 1
                if got:
                    result.append((botname, ttime, nick, userhost, txt))
                    return 1
        self.loop(logmap, None, dofunc)
        if len(result) > nrtimes:
            return result[len(result)-nrtimes:]
        else:
            return result

    def lastspoke(self, channel, userhost):
        """ return time of last line spoken by user with userhost """
        result = []
        if not channel:
            return result
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, uh, txt):
            """ add line that matches userhost """
            if userhost == uh:
                result.append(int(float(ttime)))
        self.loop(logmap, None, dofunc)
        return result[-1]

    def lastspokelist(self, channel, userhost, nrtimes):
        """ return time of last line spoken by user with userhost """
        result = []
        if not channel:
            return result
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, uh, txt):
            """ add line that matches userhost """
            if userhost == uh:
                result.append(int(float(ttime)))
        self.loop(logmap, None, dofunc)
        return result[len(result)-nrtimes:]
         
    def lastnicks(self, channel, nrtimes):
        """ return the nicks of last said lines """
        result = []
        if not channel:
            return result
        logmap = self.getmmap(channel)
        if not logmap:
            return result
        def dofunc(botname, ttime, nick, userhost, txt):
            """ add nicks  """
            result.append(nick)
        self.loop(logmap, None, dofunc)
        if len(result) > nrtimes:
            result = result[len(result)-nrtimes:]
        result.reverse()
        return result

logs = Logs('logs')

def shutdown():
    logs.save()

def prelogsay(botname, printto, txt, who, how, fromm):
    """ determine if logsay callback needs to be called """
    if printto in logs.loglist:
        return 1

def cblogsay(botname, printto, txt, who, how, fromm):
    """ log the bots output """
    logs.logbot(botname, time.time(), printto, txt)

saymonitor.add('log', cblogsay, prelogsay)

def jprelogsay(botname, msg):
    """ jabber log precondition """
    try:
        to = str(msg.getTo())
    except:
        return
    if to in logs.loglist:
        return 1

def jcblogsay(botname, msg):
    """ log the jabber message """
    try:
        to = str(msg.getTo())
        txt = msg.getBody()
    except:
        return
    logs.logbot(botname, time.time(), to, txt)

xmppmonitor.add('log', jcblogsay, jprelogsay)

def prelog(bot, ievent):
    """ log pre condition """
    if ievent.channel and ievent.channel in logs.loglist:
        return 1

def logcb(bot, ievent):
    """ callback that logs ievent """
    logs.log(bot.name, time.time(), ievent)

callbacks.add('ALL', logcb, prelog)
jcallbacks.add('Message', logcb, prelog)

def handle_logon(bot, ievent):
    """ log-on .. enable logging in channel the command was given in """
    if not ievent.channel in logs.loglist:
        logs.loglist.append(ievent.channel)
        ievent.reply('logging enabled in %s' % ievent.channel)
    else:
        ievent.reply('%s already in loglist' % ievent.channel)

cmnds.add('log-on', handle_logon, 'OPER')
examples.add('log-on', 'enable logging of the channel in which the command \
was given', 'log-on')

def handle_logoff(bot, ievent):
    """ log-off .. disable logging in channel the command was given in"""
    try:
        logs.loglist.remove(ievent.channel)
        ievent.reply('logging disabled in %s' % ievent.channel)
    except ValueError:
        ievent.reply('%s not in loglist' % ievent.channel)

cmnds.add('log-off', handle_logoff, 'OPER')
examples.add('log-off', 'disable logging of the channel in which the command \
was given', 'log-off')

def handle_loglist(bot, ievent):
    """ log-list .. show list of channels that are being logged """
    ievent.reply(str(logs.loglist))

cmnds.add('log-list', handle_loglist, 'OPER')
examples.add('log-list', 'show list of current logged channels', 'log-list')

def handle_loglen(bot, ievent):
    """ log-len show length of logfile for channel in which command was \
        given """
    if ievent.channel not in logs.loglist:
        ievent.reply('logging not enabled in %s' % ievent.channel)
        return
    if ievent.channel[0] in ['#', '!', '&', '+']:
        chan = ievent.channel[1:].lower()
    else:
        chan = ievent.channel.lower()
    ievent.reply(str(logs.size(chan)))

cmnds.add('log-len', handle_loglen, 'OPER')
examples.add('log-len', 'show size of log file of the channel the command \
was given in', 'log-len')
aliases.data['log-size'] = 'log-len'

def sayresult(bot, ievent, result, withbot=False):
    """ reply with result """
    got = 0
    if result:
        res = []
        for i in result:
            try:
                if not withbot and i[3] == 'bot@bot':
                    continue
                if not withbot and ('PRIVMSG' in i[4] or 'CMND' in i[4]):
                    what = i[4].split(':', 1)[1].strip()
                else:
                    what = i[4].strip()
                res.append("[%s %s] <%s> %s" % (dmy(float(i[1])), \
hourmin(float(i[1])), i[2], what))
                got += 1
            except (ValueError, IndexError):
                rlog(10, 'log', "can't parse %s" % i)
        ievent.reply(res, dot=True)
    if not got:
        ievent.reply("nothing found")

def handle_logback(bot, ievent):
    """ log-back [<nrtimes>] <txt> .. search back through channel log """
    if not ievent.channel:
        ievent.reply("use chan <channelname> to set channel to search in")
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging not enabled in %s' % ievent.channel)
        return
    try:
        nrtimes = int(ievent.args[0])
        txt = ' '.join(ievent.args[1:])
    except ValueError:
        txt = ievent.rest
        nrtimes = 100000
    except IndexError:
        ievent.missing('<searchitem> or <nrtimes> <searchitem>')
        return
    result = logs.back(ievent.channel, txt, nrtimes)
    sayresult(bot, ievent, result)

cmnds.add('log-back', handle_logback, ['USER', 'WEB', 'CLOUD'], speed=3)
examples.add('log-back', "log-back [<nrlinesback>] <txt> .. search backwards in log \
file of channel", '1) log-back http 2) log-back 1 http')
aliases.data['b'] = 'log-back'
aliases.data['back'] = 'log-back'

def handle_logbback(bot, ievent):
    """ log-botback [<nrtimes>] <txt> ['back' <bytesback>] .. search back \
        through bot output in the channel log """
    if not ievent.channel:
        ievent.reply("use chan <channelname> to set channel to search in")
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging not enabled in %s' % ievent.channel)
        return
    try:
        nrtimes = int(ievent.args[0])
        txt = ' '.join(ievent.args[1:])
    except ValueError:
        txt = ievent.rest
        nrtimes = 100000
    except IndexError:
        ievent.missing('<searchitem> or <nrtimes> <searchitem>')
        return
    result = logs.bback(ievent.channel, txt, nrtimes)
    sayresult(bot, ievent, result, withbot=True)

cmnds.add('log-bback', handle_logbback, ['USER', 'WEB', 'CLOUD'], speed=3)
examples.add('bback', "log-bback [<nrtimes>] <txt> ['back' <bytesback>] .. \
search backwards in the bot output of the channel log file", '1) log-back \
http 2) log-back 1 http')
aliases.data['bback'] = 'log-bback'

def handle_logsearch(bot, ievent):
    """ log-search [<nrtimes>] <txt> ['back' <bytesback>] .. search the log \
        from the beginning """
    if not ievent.channel:
        ievent.reply("use chan <channelname> to set channel to search in")
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging not enabled in %s' % ievent.channel)
        return
    try:
        nrtimes = int(ievent.args[0])
        txt = ' '.join(ievent.args[1:])
    except ValueError:
        txt = ievent.rest
        nrtimes = 100000
    except IndexError:
        ievent.missing(' <searchitem> or <nrtimes> <searchitem>')
        return
    result = logs.search(ievent.channel, txt, nrtimes)
    sayresult(bot, ievent, result)

cmnds.add('log-search', handle_logsearch, ['USER', 'WEB', 'CLOUD'], speed=3)
examples.add('log-search', 'log-search [<nrtimes>] <txt> .. search the log \
from the beginning', '1) log-search http 2) log-search 10 http')
aliasdel('search')

def handle_loglast(bot, ievent):
    """ log-last [<nr>] [<nick>] [<txt>] .. search log for last lines of
        <nick> containing <txt> """
    if ievent.channel not in logs.loglist:
        ievent.reply('logging not enabled in %s' % ievent.channel)
        return
    result = []
    try:
        nrlines = int(ievent.args[0])
        del ievent.args[0]
    except (IndexError, ValueError):
        nrlines = 100000
    try:
        nick, txt = ievent.args
    except ValueError:
        txt = None
        try:
            nick = ievent.args[0]
        except IndexError:
            nick = None
    if nick and txt:
        result = logs.linesbacknicksearch(ievent.channel, nick, txt, \
nrlines)
    elif nick: 
        result = logs.linesbacknick(ievent.channel, nick, nrlines)
    else:
        result = logs.linesback(ievent.channel, nrlines)
    if result:
        sayresult(bot, ievent, result)
    else:
        ievent.reply('no result found')

cmnds.add('log-last', handle_loglast, ['USER', 'WEB', 'CLOUD'], speed=3)
examples.add('log-last', 'log-last [<nr>] [<nick>] [<txt>] .. show lastlines \
of channel or user', '1) log-last dunker 2) log-last 5 dunker 3) log-last \
dunker http 4) log-last 5 dunker http')
aliases.data['last'] = 'log-last'

def handle_logtime(bot, ievent):
    """ show log from a certain time """
    if not logs:
        ievent.reply('log plugin is not enabled')
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging is not enabled in %s' % ievent.channel)
        return
    fromtime = strtotime(ievent.rest)
    if not fromtime:
        ievent.reply("can't detect time")
        return
    result = logs.fromtimewithbot(ievent.channel, fromtime)
    if result:
        username = users.getname(ievent.userhost)
        res = []
        for i in result:
            if i[2] == 'bot':
                txt = i[4]
            else:
                nr = i[4].find(' ')
                txt = i[4][nr:].strip()
            res.append("[%s] <%s> %s" % (hourmin(float(i[1])), i[2], txt))
        ievent.reply(res)
        return
    ievent.reply('no data found')
    
cmnds.add('log-time', handle_logtime, ['USER', 'WEB', 'CLOUD'])
examples.add('log-time', 'show log since given time', 'log-time 21:00')

def handle_active(bot, ievent):
    """ active [<minback>] .. show who has been active, default is the 
        last 15 min 
    """
    if not logs:
        ievent.reply('log plugin is not enabled')
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging is not enabled in %s' % ievent.channel)
        return
    try:
        channel = ievent.args[0].lower()
    except IndexError:
        channel = ievent.channel
    minback = 15
    result = [] 
    for i in logs.fromtime(ievent.channel, time.time() - 15*60):
        if i[2] not in result:
            result.append(i[2])
    if len(result) > 1:
        ievent.reply("active in the last %s minutes: " % minback, result)
    elif len(result) == 1:
        ievent.reply("%s is active" % result[0])
    else:
        ievent.reply("nobody active")

cmnds.add('active', handle_active, ['USER', 'WEB', 'CLOUD'])
examples.add('active', 'active [<minutesback>] .. show who has been active \
in the last 15 or <minutesback> minutes', '1) active 2) active 600')
aliases.data['a'] = 'active'

def handle_line(bot, ievent):
    """ line .. show activity of last hour """
    if not logs:
        ievent.reply('log plugin is not enabled')
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging is not enabled in %s' % ievent.channel)
        return
    now = time.time()
    times = [0]*61
    for i in logs.fromtime(ievent.channel, time.time() - 60*60):
        try:
            diff = float(now - float(i[1]))
        except ValueError:
            continue
        times[int(diff/60)] += 1
    result = ""
    for j in range(61):
        if times[j]:
            result += "%s " % times[j]
        else:
            result += "- "
    result += '(number of lines per minute for the last hour .. \
most recent minute first)'
    ievent.reply(result) 

cmnds.add('line', handle_line, ['USER', 'WEB', 'CLOUD'])
examples.add('line', 'show activity for the last hour', 'line')
aliases.data['l'] = 'line'

# thnx to timp for this one
def handle_dayline(bot, ievent):
    """ dayline .. show activity for last 24 hours """
    if not logs:
        ievent.reply('log plugin is not enabled')
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging is not enabled in %s' % ievent.channel)
        return
    now = time.time()
    times = [0]*25   
    for i in logs.fromtime(ievent.channel, time.time() - 24*60*60):
        diff = int(now/1440 - float(i[1])/1440)
        if diff < 25:
            times[diff] += 1
    result = ""
    for j in range(25):
        if times[j]:   
            result += "%s " % times[j]
        else:
            result += "- "
    result += ' (nr lines per hour for the last day .. most recent hour first)'
    ievent.reply(result)

cmnds.add('dayline', handle_dayline, ['USER', 'WEB', 'CLOUD'])
examples.add('dayline', 'show nr of lines spoken in last day', 'dl')
aliases.data['dl'] = 'dayline'

def handle_mono(bot, ievent):   
    """ mono [<nick>] .. show length of monologue """
    if not logs:
        ievent.reply('log plugin is not enabled')
        return
    if ievent.channel not in logs.loglist:
        ievent.reply('logging is not enabled in %s' % ievent.channel)
        return
    teller = 0
    skip = 0
    try:
        nick = ievent.args[0].lower() 
    except IndexError:
        nick = ievent.nick.lower()
    for i in logs.fromtime(ievent.channel, time.time() - 60*60):
        if i[4].startswith('CMND:'):
            continue
        if i[2].lower() == nick:
            teller += 1
        else:
            teller = 0
    if teller > 4:
        ievent.reply('%s lines of monologue' % teller)
    else:
        ievent.reply("%s is not making a monologue" % nick)

cmnds.add('mono', handle_mono, ['USER', 'CLOUD'])
examples.add('mono', 'mono [<nick>] .. show nr lines of monologue', \
'1) mono 2) mono dunker')



# Port from r3boot's rblcheck, with added threading support
# (c) Wijnand 'tehmaze' Modderman - http://tehmaze.com
# BSD License
# (c) Lex 'r3boot' van Roon - http://r3blog.nl

rbls = {}
rbls["wl"] = {}
rbls["bl"] = {}

rbls["wl"]["bondedsender"] = [
    "query.bondedsender.org",
    "http://www.bondedsender.org/"
]
rbls["wl"]["cml.anti-spam.org.cn"] = [
    "cml.anti-spam.org.cn",
    "http://anti-spam.org.cn/services/cml.php"
]
rbls["wl"]["dnswl"] = [
    "list.dnswl.org",
    "http://www.dnswl.org/"
]
rbls["wl"]["exemptions.ahbl"] = [
    "exemptions.ahbl.org",
    "http://www.ahbl.org/"
]
rbls["wl"]["iadb"] = [
    "iadb.isipp.com",
    "http://www.isipp.com/iadb.php"
]
rbls["wl"]["iadb2"] = [
    "iadb2.isipp.com",
    "http://www.isipp.com/iadb.php"
]
rbls["wl"]["iddb"] = [
    "iddb.isipp.com",
    "http://www.isipp.com/iadb.php"
]
rbls["wl"]["isoc"] = [
    "dnswl.isoc.bg",
    "http://dnsbl.isoc.bg/"
]
rbls["wl"]["isp.dns.nzl.net"] = [
    "isp.dns.nzl.net",
    "http://www.nzl.net/"
]
rbls["wl"]["nlwhitelist"] = [
    "nlwhitelist.dnsbl.bit.nl",
    "http://virbl.bit.nl/"
]
rbls["wl"]["spamblocked"] = [
    "whitelist.spamblocked.com",
    "http://www.spamblocked.com/"
]
rbls["wl"]["spf.trusted-forwarder.org"] = [
    "spf.trusted-forwarder.org",
    "http://www.trusted-forwarder.org/"
]
rbls["wl"]["whitelist.sci.kun.nl"] = [
    "whitelist.sci.kun.nl",
    "http://whitelist.sci.kun.nl"
]
rbls["bl"]["access.redhawk.org"] = [
    "access.redhawk.org",
    "http://access.redhawk.org"
]
rbls["bl"]["assholes.madscience.nl"] = [
    "assholes.madscience.nl",
    "http://assholes.madscience.nl"
]
rbls["bl"]["badconf.rhsbl.sorbs.net"] = [
    "badconf.rhsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["bl.csma.biz"] = [
    "bl.csma.biz",
    "http://bl.csma.biz/"
]
rbls["bl"]["bl.deadbeef.com"] = [
    "bl.deadbeef.com",
    "http://spam.deadbeef.com/bl/"
]
rbls["bl"]["bl.spamcannibal.org"] = [
    "bl.spamcannibal.org",
    "http://spamcannibal.org/"
]
rbls["bl"]["bl.starloop.com"] = [
    "bl.starloop.com",
    "http://bl.starloop.com"
]
rbls["bl"]["bl.technovision.dk"] = [
    "bl.technovision.dk",
    "http://bl.technovision.dk"
]
rbls["bl"]["blackholes.five-ten-sg.com"] = [
    "blackholes.five-ten-sg.com",
    "http://www.five-ten-sg.com/blackhole.php"
]
rbls["bl"]["blackholes.intersil.net"] = [
    "blackholes.intersil.net",
    "http://blackholes.intersil.net"
]
rbls["bl"]["blackholes.mail-abuse.org"] = [
    "blackholes.mail-abuse.org",
    "http://www.mail-abuse.org/rss/"
]
rbls["bl"]["blackholes.sandes.dk"] = [
    "blackholes.sandes.dk",
    "http://blackholes.sandes.dk"
]
rbls["bl"]["blackholes.uceb.org"] = [
    "blackholes.uceb.org",
    "http://www.uceb.org"
]
rbls["bl"]["blacklist.sci.kun.nl"] = [
    "blacklist.sci.kun.nl",
    "http://blacklist.sci.kun.nl"
]
rbls["bl"]["blacklist.spambag.org"] = [
    "blacklist.spambag.org",
    "http://blacklist.spambag.org"
]
rbls["bl"]["block.dnsbl.sorbs.net"] = [
    "block.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["blocked.hilli.dk"] = [
    "blocked.hilli.dk",
    "http://blocked.hilli.dk"
]
rbls["bl"]["bogons.dnsiplists.completewhois.com"] = [
    "bogons.dnsiplists.completewhois.com",
    "http://bogons.dnsiplists.completewhois.com"
]
rbls["bl"]["cart00ney.surriel.com"] = [
    "cart00ney.surriel.com",
    "http://cart00ney.surriel.com"
]
rbls["bl"]["cbl.abuseat.org"] = [
    "cbl.abuseat.org",
    "http://cbl.abuseat.org"
]
rbls["bl"]["dev.null.dk"] = [
    "dev.null.dk",
    "http://dev.null.dk/"
]
rbls["bl"]["dialup.blacklist.jippg.org"] = [
    "dialup.blacklist.jippg.org",
    "http://dialup.blacklist.jippg.org"
]
rbls["bl"]["dialups.mail-abuse.org"] = [
    "dialups.mail-abuse.org",
    "http://www.mail-abuse.org/dul/"
]
rbls["bl"]["dialups.visi.com"] = [
    "dialups.visi.com",
    "http://dialups.visi.com"
]
rbls["bl"]["dnsbl-1.uceprotect.net"] = [
    "dnsbl-1.uceprotect.net",
    "http://www.uceprotect.net/en/"
]
rbls["bl"]["dnsbl-2.uceprotect.net"] = [
    "dnsbl-2.uceprotect.net",
    "http://www.uceprotect.net/en/"
]
rbls["bl"]["dnsbl-3.uceprotect.net"] = [
    "dnsbl-3.uceprotect.net",
    "http://www.uceprotect.net/en/"
]
rbls["bl"]["dnsbl.ahbl.org"] = [
    "dnsbl.ahbl.org",
    "http://www.ahbl.org/"
]
rbls["bl"]["dnsbl.antispam.or.id"] = [
    "dnsbl.antispam.or.id",
    "http://dnsbl.antispam.or.id"
]
rbls["bl"]["dnsbl.cyberlogic.net"] = [
    "dnsbl.cyberlogic.net",
    "http://dnsbl.cyberlogic.net"
]
rbls["bl"]["dnsbl.jammconsulting.com"] = [
    "dnsbl.jammconsulting.com",
    "http://www.jammconsulting.com/policies/dnsbl.shtml"
]
rbls["bl"]["dnsbl.kempt.net"] = [
    "dnsbl.kempt.net",
    "http://dnsbl.kempt.net"
]
rbls["bl"]["dnsbl.njabl.org"] = [
    "dnsbl.njabl.org",
    "http://dnsbl.njabl.org"
]
rbls["bl"]["dnsbl.sorbs.net"] = [
    "dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["dsbl.dnsbl.net.au"] = [
    "dsbl.dnsbl.net.au",
    "http://dsbl.dnsbl.net.au"
]
rbls["bl"]["duinv.aupads.org"] = [
    "duinv.aupads.org",
    "http://duinv.aupads.org"
]
rbls["bl"]["dul.dnsbl.sorbs.net"] = [
    "dul.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["dul.maps.vix.com"] = [
    "dul.maps.vix.com",
    "http://dul.maps.vix.com/"
]
rbls["bl"]["dul.orca.bc.ca"] = [
    "dul.orca.bc.ca",
    "http://dul.orca.bc.ca/"
]
rbls["bl"]["dul.ru"] = [
    "dul.ru",
    "http://dul.ru"
]
rbls["bl"]["dun.dnsrbl.net"] = [
    "dun.dnsrbl.net",
    "http://dun.dnsrbl.net"
]
rbls["bl"]["dynablock.njabl.org"] = [
    "dynablock.njabl.org",
    "http://dynablock.njabl.org"
]
rbls["bl"]["fl.chickenboner.biz"] = [
    "fl.chickenboner.biz",
    "http://fl.chickenboner.biz"
]
rbls["bl"]["forbidden.icm.edu.pl"] = [
    "forbidden.icm.edu.pl",
    "http://forbidden.icm.edu.pl"
]
rbls["bl"]["hijacked.dnsiplists.completewhois.com"] = [
    "hijacked.dnsiplists.completewhois.com",
    "http://hijacked.dnsiplists.completewhois.com"
]
rbls["bl"]["hil.habeas.com"] = [
    "hil.habeas.com",
    "http://hil.habeas.com"
]
rbls["bl"]["http.dnsbl.sorbs.net"] = [
    "http.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["images-msrbls"] = [
    "images.rbl.msrbl.net",
    "http://www.msrbl.com/"
]
rbls["bl"]["intruders.docs.uu.se"] = [
    "intruders.docs.uu.se",
    "http://intruders.docs.uu.se"
]
rbls["bl"]["korea.services.net"] = [
    "korea.services.net",
    "http://korea.services.net"
]
rbls["bl"]["l1.spews.dnsbl.sorbs.net"] = [
    "l1.spews.dnsbl.sorbs.net",
    "http://l1.spews.dnsbl.sorbs.net"
]
rbls["bl"]["l2.spews.dnsbl.sorbs.net"] = [
    "l2.spews.dnsbl.sorbs.net",
    "http://l2.spews.dnsbl.sorbs.net"
]
rbls["bl"]["mail-abuse.blacklist.jippg.org"] = [
    "mail-abuse.blacklist.jippg.org",
    "http://mail-abuse.blacklist.jippg.org"
]
rbls["bl"]["map.spam-rbl.com"] = [
    "map.spam-rbl.com",
    "http://map.spam-rbl.com"
]
rbls["bl"]["misc.dnsbl.sorbs.net"] = [
    "misc.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["msgid.bl.gweep.ca"] = [
    "msgid.bl.gweep.ca",
    "http://msgid.bl.gweep.ca"
]
rbls["bl"]["msrbl"] = [
    "combined.rbl.msrbl.net",
    "http://www.msrbl.com/"
]
rbls["bl"]["no-more-funn.moensted.dk"] = [
    "no-more-funn.moensted.dk",
    "http://moensted.dk/spam/no-more-funn/"
]
rbls["bl"]["nomail.rhsbl.sorbs.net"] = [
    "nomail.rhsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["ohps.dnsbl.net.au"] = [
    "ohps.dnsbl.net.au",
    "http://ohps.dnsbl.net.au"
]
rbls["bl"]["okrelays.nthelp.com"] = [
    "okrelays.nthelp.com",
    "http://okrelays.nthelp.com"
]
rbls["bl"]["omrs.dnsbl.net.au"] = [
    "omrs.dnsbl.net.au",
    "http://omrs.dnsbl.net.au"
]
rbls["bl"]["orid.dnsbl.net.au"] = [
    "orid.dnsbl.net.au",
    "http://orid.dnsbl.net.au"
]
rbls["bl"]["orvedb.aupads.org"] = [
    "orvedb.aupads.org",
    "http://orvedb.aupads.org"
]
rbls["bl"]["osps.dnsbl.net.au"] = [
    "osps.dnsbl.net.au",
    "http://osps.dnsbl.net.au"
]
rbls["bl"]["osrs.dnsbl.net.au"] = [
    "osrs.dnsbl.net.au",
    "http://osrs.dnsbl.net.au"
]
rbls["bl"]["owfs.dnsbl.net.au"] = [
    "owfs.dnsbl.net.au",
    "http://owfs.dnsbl.net.au"
]
rbls["bl"]["owps.dnsbl.net.au"] = [
    "owps.dnsbl.net.au",
    "http://owps.dnsbl.net.au"
]
rbls["bl"]["pdl.dnsbl.net.au"] = [
    "pdl.dnsbl.net.au",
    "http://pdl.dnsbl.net.au"
]
rbls["bl"]["phishing-msrbl"] = [
    "phishing.rbl.msrbl.net",
    "http://www.msrbl.com/"
]
rbls["bl"]["probes.dnsbl.net.au"] = [
    "probes.dnsbl.net.au",
    "http://probes.dnsbl.net.au"
]
rbls["bl"]["proxy.bl.gweep.ca"] = [
    "proxy.bl.gweep.ca",
    "http://proxy.bl.gweep.ca"
]
rbls["bl"]["psbl.surriel.com"] = [
    "psbl.surriel.com",
    "http://psbl.surriel.com"
]
rbls["bl"]["pss.spambusters.org.ar"] = [
    "pss.spambusters.org.ar",
    "http://pss.spambusters.org.ar"
]
rbls["bl"]["rbl-plus.mail-abuse.org"] = [
    "rbl-plus.mail-abuse.org",
    "http://rbl-plus.mail-abuse.org"
]
rbls["bl"]["rbl.cluecentral.net"] = [
    "rbl.cluecentral.net",
    "http://rbl.cluecentral.net"
]
rbls["bl"]["rbl.efnet.org"] = [
    "rbl.efnet.org",
    "http://rbl.efnet.org"
]
rbls["bl"]["rbl.jp"] = [
    "rbl.jp",
    "http://rbl.jp"
]
rbls["bl"]["rbl.maps.vix.com"] = [
    "rbl.maps.vix.com",
    "http://www.mail-abuse.org/rbl/"
]
rbls["bl"]["rbl.schulte.org"] = [
    "rbl.schulte.org",
    "http://rbl.schulte.org"
]
rbls["bl"]["rbl.snark.net"] = [
    "rbl.snark.net",
    "http://rbl.snark.net"
]
rbls["bl"]["rbl.triumf.ca"] = [
    "rbl.triumf.ca",
    "http://rbl.triumf.ca"
]
rbls["bl"]["rdts.dnsbl.net.au"] = [
    "rdts.dnsbl.net.au",
    "http://rdts.dnsbl.net.au"
]
rbls["bl"]["relays.bl.gweep.ca"] = [
    "relays.bl.gweep.ca",
    "http://relays.bl.gweep.ca"
]
rbls["bl"]["relays.bl.kundenserver.de"] = [
    "relays.bl.kundenserver.de",
    "http://relays.bl.kundenserver.de"
]
rbls["bl"]["relays.mail-abuse.org"] = [
    "relays.mail-abuse.org",
    "http://work-rss.mail-abuse.org/rss/"
]
rbls["bl"]["relays.nether.net"] = [
    "relays.nether.net",
    "http://relays.nether.net"
]
rbls["bl"]["relays.nthelp.com"] = [
    "relays.nthelp.com",
    "http://relays.nthelp.com"
]
#rbls["bl"]["relays.radparker.com"] = [
#    "relays.radparker.com",
#    "http://relays.radparker.com/"
#]
rbls["bl"]["rhsbl.sorbs.net"] = [
    "rhsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["ricn.dnsbl.net.au"] = [
    "ricn.dnsbl.net.au",
    "http://ricn.dnsbl.net.au"
]
rbls["bl"]["rmst.dnsbl.net.au"] = [
    "rmst.dnsbl.net.au",
    "http://rmst.dnsbl.net.au"
]
rbls["bl"]["rsbl.aupads.org"] = [
    "rsbl.aupads.org",
    "http://rsbl.aupads.org"
]
rbls["bl"]["satos.rbl.cluecentral.net"] = [
    "satos.rbl.cluecentral.net",
    "http://satos.rbl.cluecentral.net"
]
rbls["bl"]["sbl-xbl.spamhaus.org"] = [
    "sbl-xbl.spamhaus.org",
    "http://sbl-xbl.spamhaus.org"
]
rbls["bl"]["sbl.csma.biz"] = [
    "sbl.csma.biz",
    "http://bl.csma.biz/"
]
rbls["bl"]["sbl.spamhaus.org"] = [
    "sbl.spamhaus.org",
    "http://www.spamhaus.org/sbl/"
]
rbls["bl"]["smtp.dnsbl.sorbs.net"] = [
    "smtp.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["socks.dnsbl.sorbs.net"] = [
    "socks.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["sorbs.dnsbl.net.au"] = [
    "sorbs.dnsbl.net.au",
    "http://sorbs.dnsbl.net.au"
]
rbls["bl"]["spam.dnsbl.sorbs.net"] = [
    "spam.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["spam.dnsrbl.net"] = [
    "spam.dnsrbl.net",
    "http://spam.dnsrbl.net"
]
rbls["bl"]["spam.olsentech.net"] = [
    "spam.olsentech.net",
    "http://spam.olsentech.net"
]
rbls["bl"]["spam.wytnij.to"] = [
    "spam.wytnij.to",
    "http://spam.wytnij.to"
]
rbls["bl"]["spamcop"] = [
    "bl.spamcop.net",
    "http://www.spamcop.net/bl.shtml"
]
rbls["bl"]["spamguard.leadmon.net"] = [
    "spamguard.leadmon.net",
    "http://spamguard.leadmon.net"
]
rbls["bl"]["spamsites.dnsbl.net.au"] = [
    "spamsites.dnsbl.net.au",
    "http://spamsites.dnsbl.net.au"
]
rbls["bl"]["spamsources.dnsbl.info"] = [
    "spamsources.dnsbl.info",
    "http://spamsources.dnsbl.info"
]
rbls["bl"]["spamsources.fabel.dk"] = [
    "spamsources.fabel.dk",
    "http://spamsources.fabel.dk"
]
rbls["bl"]["spews.dnsbl.net.au"] = [
    "spews.dnsbl.net.au",
    "http://spews.dnsbl.net.au"
]
rbls["bl"]["t1.dnsbl.net.au"] = [
    "t1.dnsbl.net.au",
    "http://www.dnsbl.net.au/t1/"
]
rbls["bl"]["ucepn.dnsbl.net.au"] = [
    "ucepn.dnsbl.net.au",
    "http://ucepn.dnsbl.net.au"
]
rbls["bl"]["virbl"] = [
    "virbl.dnsbl.bit.nl",
    "https://virbl.bit.nl/faq.php"
]
rbls["bl"]["virus-msrbl"] = [
    "virus.rbl.msrbl.net",
    "http://www.msrbl.com/"
]
rbls["bl"]["virus.rbl.jp"] = [
    "virus.rbl.jp",
    "http://virus.rbl.jp"
]
rbls["bl"]["web.dnsbl.sorbs.net"] = [
    "web.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]
rbls["bl"]["whois.rfc-ignorant.org"] = [
    "whois.rfc-ignorant.org",
    "http://whois.rfc-ignorant.org"
]
rbls["bl"]["will-spam-for-food.eu.org"] = [
    "will-spam-for-food.eu.org",
    "http://will-spam-for-food.eu.org"
]
rbls["bl"]["xbl.spamhaus.org"] = [
    "xbl.spamhaus.org",
    "http://www.spamhaus.org/xbl/"
]
rbls["bl"]["zombie.dnsbl.sorbs.net"] = [
    "zombie.dnsbl.sorbs.net",
    "http://www.dnsbl.nl.sorbs.net/"
]

from socket    import gethostbyaddr
from socket    import gethostbyname, gaierror
from time      import time, sleep
from Queue     import Queue, Empty
from threading import Thread
import re
from gozerbot.commands      import cmnds
from gozerbot.generic       import rlog
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.users         import users
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp

plughelp.add('rblcheck', 'check blacklists')

cfg = PersistConfig()
cfg.define('throttle', 60)
cfg.define('max-threads', 5)
cfg.define('debug', 0)
debug = 0

def list_check(list, chkip):
    results = []
    for i in list.keys():
        chk = "%s.%s" % (".".join(reversed(chkip.split("."))), list[i][0])
        try: qr = gethostbyname("%s" % (chk))
        except: qr = None
        if qr:
            results.append(list[i])
            if cfg.get('debug'):
                rlog(10, 'rblcheck', '%s result: %s' % (chk, str(qr)))
    return results

class pool_check(object):

    timeout = 1.0

    def __init__(self, lookups, ip):
        self.cip = ip
        self.rip = ".".join(reversed(self.cip.split(".")))
        self.todo = Queue()
        self.hits = Queue()
        self.pool = []
        self.fill(lookups)

    def fill(self, lookups):
        # propagate todo queue
        for lookup in lookups:
            self.todo.put([lookup, lookups[lookup]])

    def run(self):
        # start runner threads
        for i in xrange(0, cfg.get('max-threads')):
            runner = Thread(target=self.poll)
            runner.start()
            self.pool.append(runner)
        # now we wait until all runners in the pool are done
        while True:
            if not self.pool:
                break
            sleep(self.timeout)
        hits = []
        while True:
            try: hits.append(self.hits.get(block=False, timeout=0.01))
            except Empty: break
        return hits

    def poll(self):
        while True:
            try:
                item, info = self.todo.get(block=False, timeout=self.timeout)
                try: listed = gethostbyname("%s.%s" % (self.rip, item))
                except gaierror: listed = False
                if listed: self.hits.put(info)
            except Empty:
                break # no more items in queue
        self.pool.pop(0) # remove item from queue

class RBLCheck(object):
    
    re_ip = re.compile('^(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$')
    throttle = {}

    def handle_rblcheck(self, bot, ievent):
        if not self.re_ip.match(ievent.rest.strip()):
            ievent.missing('<ip>')
            return
        user = users.getname(ievent.userhost)
        if self.throttle.has_key(user) and self.throttle[user] >= time():
            ievent.reply('throttled - you can only do one lookup each %d seconds' % cfg.get('throttle'))
            return
        self.throttle[user] = time() + cfg.get('throttle')
        ip = ievent.rest.strip()
        ievent.reply('checking %d blacklists' % len(rbls['bl']))
        checks = pool_check(rbls['bl'], ip)
        result = checks.run()
        #result = list_check(rbls['bl'], ip)
        if result:
            results = []
            for r in result:
                results.append('%s (%s)' % (r[0], r[1]))
            ievent.reply('%s is blacklisted: ' % ip, results)
        else:
            ievent.reply('%s is not blacklisted' % ip)

rblcheck = RBLCheck()
cmnds.add('rblcheck', rblcheck.handle_rblcheck, 'USER', threaded=True)
examples.add('rblcheck', 'check <ip> with blacklists', 'rblcheck 127.0.0.1')




# plugs/tinyurl.py
#
#

""" tinyurl.com feeder """

__author__ = "Wijnand 'tehmaze' Modderman - http://tehmaze.com"
__license__ = 'BSD'
__depending__ = ['rss', ]

from gozerbot.aliases import aliases
from gozerbot.callbacks import callbacks
from gozerbot.commands import cmnds
from gozerbot.generic import striphtml, useragent, rlog
from gozerbot.plughelp import plughelp
from gozerbot.examples import examples
import urllib
import urllib2
import urlparse
import re

plughelp.add('tinyurl', 'the tinyurl url provides a tiny url for the url \
provided as argument or the last url in the log')

re_url_match  = re.compile(u'((?:http|https)://\S+)')
urlcache = {}

def valid_url(url):
    """ check if url is valid """
    if not re_url_match.search(url):
        return False
    parts = urlparse.urlparse(url)
    cleanurl = '%s://%s' % (parts[0], parts[1])
    if parts[2]:
        cleanurl = '%s%s' % (cleanurl, parts[2])
    if parts[3]:
        cleanurl = '%s;%s' % (cleanurl, parts[3])
    if parts[4]:
        cleanurl = '%s?%s' % (cleanurl, parts[4])
    return cleanurl


def precb(bot, ievent):
    test_url = re_url_match.search(ievent.txt)
    if test_url:
        return 1

def privmsgcb(bot, ievent):
    """ callback for urlcaching """
    test_url = re_url_match.search(ievent.txt)
    if test_url:
        url = test_url.group(1)
        if not urlcache.has_key(bot.name):
            urlcache[bot.name] = {}
        urlcache[bot.name][ievent.target] = url

callbacks.add('PRIVMSG', privmsgcb, precb)

def get_tinyurl(url):
    """ grab a tinyurl """
    postarray = [
        ('submit', 'submit'),
        ('url', url),
        ]
    postdata = urllib.urlencode(postarray)
    req = urllib2.Request(url='http://tinyurl.com/create.php', data=postdata)
    req.add_header('User-agent', useragent())
    try:
        res = urllib2.urlopen(req).readlines()
        #raise Exception("mekker")
    except urllib2.URLError, e:
        rlog(10, 'tinyurl', 'URLError: %s' % str(e))
        return
    except urllib2.HTTPError, e:
        rlog(10, 'tinyurl', 'HTTP error: %s' % str(e))
        return
    urls = []
    for line in res:
        if line.startswith('<blockquote><b>'):
            urls.append(striphtml(line.strip()).split('[Open')[0])
    if len(urls) == 3:
        urls.pop(0)
    return urls

def handle_tinyurl(bot, ievent):
    """ get tinyurl from argument or last url in log """
    if not ievent.rest and (not urlcache.has_key(bot.name) or not \
urlcache[bot.name].has_key(ievent.target)):
        ievent.missing('<url>')
        return
    elif not ievent.rest:
        url = urlcache[bot.name][ievent.target]
    else:
        url = ievent.rest
    url = valid_url(url)
    if not url:
        ievent.reply('invalid or bad URL')
        return
    tinyurl = get_tinyurl(url)
    if tinyurl:
        ievent.reply(' .. '.join(tinyurl))
    else:
        ievent.reply('failed to create tinyurl')

cmnds.add('tinyurl', handle_tinyurl, 'USER', threaded=True)
examples.add('tinyurl', 'show a tinyurl', 'tinyurl http://gozerbot.org')
aliases.data['tu'] = 'tinyurl'



import hashlib
import os

from gozerbot.commands import cmnds
from gozerbot.datadir import datadir
from gozerbot.fleet import fleet
from gozerbot.generic import geturl2
from gozerbot.periodical import periodical
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.persist.pdod import Pdod
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import gozerbot.threads.thr as thr

plughelp.add('httpwatch', 'periodically check urls')

cfg = PersistConfig()
cfg.define('sleep', 300)
cfg.define('run', True)

class HttpWatch(Pdod):
    
    pid = None

    def __init__(self):
        self.datadir = datadir + os.sep + 'plugs' + os.sep + 'httpwatch'
        Pdod.__init__(self, os.path.join(self.datadir, 'httpwatch.data'))
        if not 'urls' in self.data:
            self.data['urls'] = {}
        if not 'send' in self.data:
            self.data['send'] = {}
            self.save()

    def start(self):
        self.pid = periodical.addjob(cfg.get('sleep'), 0, self.peek, 'httpwatch', 'httpwatch')
 
    def stop(self):
        if self.pid:
            periodical.killjob(self.pid)
            self.pid = None

    def peek(self, *args, **kwargs):
        if not self.data['urls']: return
        for url in self.data['urls']:
            try:
                checksum = self.checksum(url)
            except:
                continue
            
            if checksum != self.data['urls'][url]:
                self.data['urls'][url] = checksum
                self.announce(url)
        self.save()
        return self.data['urls']

    def checksum(self, url):
        data = geturl2(url)
        return hashlib.md5.new(data).hexdigest()

    def announce(self, url):
        for name in self.data['send'][url]:
            bot = fleet.byname(name)
            for chan in self.data['send'][url][name]:
                bot.say(chan, '%s changed (new checksum: %s)' % (url, \
self.data['urls'][url]))

    def add(self, url, name, chan):
        if not url in self.data['urls']:
            self.data['urls'][url] = ''
        if not url in self.data['send']:
            self.data['send'][url] = {}
        if not name in self.data['send'][url]:
            self.data['send'][url][name] = []
        if not chan in self.data['send'][url][name]:
            self.data['send'][url][name].append(chan)
            self.save()

    def remove(self, url, name, chan):
        if url not in self.data['send']:
            return
        if name not in self.data['send'][url]:
            return
        if chan in self.data['send'][url][name]:
            self.data['send'][url][name].remove(chan)
        if len(self.data['send'][url][name]) == 0:
            del self.data['send'][url][name]
        if len(self.data['send'][url]) == 0:
            del self.data['send'][url]
        self.save()

httpwatch = HttpWatch()

def init():
    if cfg.get('run'):
        thr.start_new_thread(httpwatch.start, ())
    return 1

def shutdown():
    if httpwatch.pid:
        httpwatch.stop()
    return 1

def handle_httpwatch_add(bot, ievent):
    if len(ievent.args) != 1:
        ievent.missing('<url>')
    else:
        try:
            geturl2(ievent.args[0])
            httpwatch.add(ievent.args[0], bot.name, ievent.channel)
            ievent.reply('http watcher added')
        except Exception, e:
            ievent.reply('failed to add: %s' % (str(e),))

def handle_httpwatch_del(bot, ievent):
    if len(ievent.args) != 1:
        ievent.missing('<url>')
    else:
        httpwatch.remove(ievent.args[0], bot.name, ievent.channel)
        ievent.reply('http watcher removed')

def handle_httpwatch_list(bot, ievent):
    ievent.reply(str(httpwatch.data['send']))

def handle_httpwatch_peek(bot, ievent):
    ievent.reply('running httpwatch.peek()')
    ievent.reply(str(httpwatch.peek()))

def handle_httpwatch_start(bot, ievent):
    if httpwatch.pid:
        ievent.reply('watcher already running (pid: %s)' % (str(httpwatch.pid),))
    else:
        httpwatch.start()
        ievent.reply('watcher started')

def handle_httpwatch_stop(bot, ievent):
    if httpwatch.pid:
        httpwatch.stop()
        ievent.reply('watcher stopped')
    else:
        ievent.reply('watcher not running')

cmnds.add('httpwatch-add', handle_httpwatch_add, 'OPER')
cmnds.add('httpwatch-del', handle_httpwatch_del, 'OPER')
cmnds.add('httpwatch-list', handle_httpwatch_list, 'OPER')
cmnds.add('httpwatch-peek', handle_httpwatch_peek, 'OPER')
cmnds.add('httpwatch-start', handle_httpwatch_start, 'OPER')
cmnds.add('httpwatch-stop', handle_httpwatch_stop, 'OPER')




# gozerplugs/chatlog.py
#

""" log irc channels to [hour:min] <nick> txt format 
The db backend expects the following table structure
$dbtable(time, network, target, nick, type, message)
"""

__copyright__ = 'this file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.callbacks import callbacks, jcallbacks
from gozerbot.database.alchemy import dbstart, create_session, trans, create_all
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.generic import hourmin, rlog, lockdec
from gozerbot.irc.monitor import outmonitor
from gozerbot.xmpp.monitor import xmppmonitor
from gozerbot.plughelp import plughelp
from gozerbot.examples import examples
from gozerbot.irc.ircevent import Ircevent
from gozerbot.fleet import fleet
import time, os, thread
from os import path
from datetime import datetime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, Text, DateTime, Integer, Sequence

IrclogsBase = declarative_base()

class ChatLog(IrclogsBase):
    __tablename__ = 'chatlog'
    __table_args__ = {'useexisting': True}
    id = Column('id', Integer, Sequence('chatlog_id_seq', optional=True), primary_key=True)
    time = Column('time', DateTime, nullable=False, default=datetime.now)
    network = Column('network', String(256), nullable=False, default='')
    target = Column('target', String(256), nullable=False, default='')
    nick = Column('nick', String(256), nullable=False, default='')
    type = Column('type', String(256), nullable=False, default='')
    message = Column('message', Text, nullable=False, default='')

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


plughelp.add('chatlog', 'log irc channels')

outlock = thread.allocate_lock()
outlocked = lockdec(outlock)

cfg = PersistConfig()
cfg.define('channels', [])
# format for logs. simple or supy are currently supported.
cfg.define('format', 'simple')
# basepath: the root of all logs.  None is $gozerbot-root/logs
cfg.define('basepath', '')
# msgs that should be logged start with this
cfg.define('nologprefix', '[nolog]')
# and are replaced with
cfg.define('nologmsg', '-= THIS MESSAGE NOT LOGGED =-')
# where do we write the logs too?
cfg.define('backend', 'file')
# db settings for db backend
cfg.define('dbtype', '')
cfg.define('dbname', '')
cfg.define('dbhost', '')
cfg.define('dbuser', '')
cfg.define('dbpasswd', '')
cfg.define('dburi', '')
logfiles = {}
backends = {}
stopped = False
db = None

 # Formats are defined here. simple also provides default values if values
 # are not supplied by the format, as well as format 'simple'. 
 # Parameters that should be supplied:
 #   * timestamp_format: format of timestamp in log files
 #     * all strftime vars supported.
 #   * filename: file name for log
 #     * var channel : full channel ie. #dunkbot
 #     * var channel_name : channel without '#' ie. dunkbot
 #   * event_filename: 
 #        if event_filename exists, then it will be used for
 #        logging events (seperate from chat)
 #     * var channel : full channel ie. #dunkbot
 #     * var channel_name : channel without '#' ie. dunkbot
 #   * separator: the separator between the timestamp and message
formats = {
    'simple': {
        'timestamp_format': '%Y-%m-%d %H:%M:%S',
        'basepath': None,
        'filename': 'logs/%%(network)s/simple/%%(target)s.%Y%m%d.slog',
        'event_prefix': '',
        'event_filename': 'logs/%%(network)s/simple/%%(channel_name)s.%Y%m%d.slog',
        'separator': ' | ',
    },
    'supy': {
        'timestamp_format': '%Y-%m-%dT%H:%M:%S',
        'filename': 'logs/%%(network)s/supy/%%(target)s/%%(target)s.%Y-%m-%d.log',
        'event_prefix': '*** ',
        'event_filename': None,
        'separator': '  ',
    }
}

# Get a format opt in the currently cfg'd format
def format_opt(name):
    simple_format = formats['simple']
    format = formats.get(cfg.get('format'), 'simple')
    opt = format.get(name, simple_format.get(name, None))
    return opt

def init():
    global stopped
    stopped = False
    callbacks.add('ALL', chatlogcb, prechatlogcb)
    jcallbacks.add('ALL', jabberchatlogcb, jabberprechatlogcb)
    outmonitor.add('chatlog', chatlogcb, prechatlogcb)
    xmppmonitor.add('chatlog-jabber', jabberchatlogcb, jabberprechatlogcb)
    return 1

def shutdown():
    global stopped
    stopped = True
    for file in logfiles.values():
        file.close()
    return 1

def timestr():
    return time.strftime(format_opt('timestamp_format'))

def write(m): 
    """m is a dict with the following properties:
      timestamp
      type : (comment, nick, topic etc..)
      target : (#channel, bot etc..)
      txt : actual message
      network
    """
    backend_name = cfg.get('backend', 'file')
    backend = backends.get(backend_name, file_write)
    if m['txt'].startswith(cfg.get('nologprefix')):
        m['txt'] = cfg.get('nologmsg')
    backend(m)

def file_write(m):
    if stopped:
        return
    args = {
        'target': m.get('target'), 
        'network': m.get('network'), 
    }
    if args['target'].startswith('#'):
        args['channel_name'] = args['target'][1:]
    f = time.strftime(format_opt('filename')) % args
    # if this is an event, and there is an event_filename, use that
    # instead of filename
    if m['type'] != 'comment':
        event_filename = format_opt('event_filename')
        if event_filename:
            f = time.strftime(event_filename % args)
        m['txt'] = '%s%s'%(m['event_prefix'], m['txt'])
    else:
        m['txt'] = '<%s> %s'%(m['nick'], m['txt'])

    # if there is a basepath specified, append it,
    # else it should go to a dir relative to the 
    # gozerbot dir.
    basepath = cfg.get('basepath')
    if basepath:
        f = path.join(basepath, f)

    # create dir if it doesn't exist
    dir = path.dirname(f)
    if not path.exists(dir):
        os.makedirs(dir)

    line = '%(timestamp)s%(separator)s%(txt)s\n'%({
        'timestamp': m['timestamp'], 
        'separator': format_opt('separator'),
        'txt': m['txt'],
    })

    try:
        if logfiles.has_key(f):
            logfiles[f].write(line)
            logfiles[f].flush()
        else:
            rlog(5, 'chatlog', 'opening %s for logging'%(f))
            logfiles[f] = open(f, 'a')
            logfiles[f].write(line)
            logfiles[f].flush()
    except Exception, ex:
        rlog(10, 'chatlog', str(ex))

backends['file'] = file_write

def init_db():
    global db
    try:
        session, engine = dbstart(mainconfig=cfg, base=IrclogsBase) 
    except:
        session, engine = dbstart(base=IrclogsBase) 
    db = session
    create_all('chatlog', base=IrclogsBase)
    

def db_write(m):
    try:
        global db
        db or init_db()
        row = {
                'time' : m.get('timestamp'),
                'network' : m.get('network'),
                'target' : m.get('target'),
                'nick' : m.get('nick'),
                'type' : m.get('type'),
                'message' : m.get('txt')
        }
        db.add(ChatLog(**row))
        db.commit()
    except Exception, e:
        rlog(10, 'chatlog', 'failed to log to db: %s'%(str(e)))
    
backends['db'] = db_write

def log(bot, ievent):
    m = {
        'timestamp': timestr(),
        'separator': format_opt('separator'),
        'event_prefix': format_opt('event_prefix'),
        'network': bot.networkname,
        'nick': ievent.nick,
        'target': ievent.channel,
    }

    if ievent.cmnd == 'PRIVMSG':
        if ievent.txt.startswith('\001ACTION'):
            m.update({
                'type': 'action',
                'txt': '* %s %s'%(m['nick'], ievent.txt[7:-1].strip()),
            })
        else:
            m.update({
                'type': 'comment',
                'txt': '%s'%(ievent.origtxt),
            })
    elif ievent.cmnd == 'NOTICE':
        m.update({
            'type': 'notice',
            'target': ievent.arguments[0],
            'txt': "-%s- %s"%(ievent.nick, ievent.txt)
        })
    elif ievent.cmnd == 'TOPIC':
        m.update({
            'type': 'topic',
            'txt': '%s changes topic to "%s"'%(ievent.nick, ievent.txt),
        })
    elif ievent.cmnd == 'MODE':
        margs = ' '.join(ievent.arguments[1:])
        m.update({
            'type': 'mode',
            'txt': '%s sets mode: %s'%(ievent.nick, margs),
        })
    elif ievent.cmnd == 'JOIN':
        m.update({
            'type': 'join',
            'txt': '%s (%s) has joined'%(ievent.nick, ievent.userhost),
        })
    elif ievent.cmnd == 'KICK':
        m.update({
            'type': 'kick',
            'txt': '%s was kicked by %s (%s)'%(
                ievent.arguments[1], 
                ievent.nick, 
                ievent.txt
            ),
        })
    elif ievent.cmnd == 'PART':
        m.update({
            'type': 'part',
            'txt': '%s (%s) has left'%(ievent.nick, ievent.userhost),
        })
    elif ievent.cmnd in ('QUIT', 'NICK'):
        cmd = ievent.cmnd
        nick = cmd == 'NICK' and ievent.txt or ievent.nick
        if not bot.userchannels.has_key(nick.lower()):
            return
        for c in bot.userchannels[nick.lower()]:
            if [bot.name, c] in cfg.get('channels'):
                if c in bot.state['joinedchannels']:
                    if cmd == 'NICK':
                        m['txt'] = '%s (%s) is now known as %s'%(
                            ievent.nick, ievent.userhost, ievent.txt
                        )
                    else:
                        m['txt'] = '%s (%s) has quit: %s'%(
                            ievent.nick, ievent.userhost, ievent.txt
                        )
                    m.update({
                        'type': ievent.cmnd.lower(),
                        'target': c,
                    })
                    write(m)
        return
    if m.get('txt'):
        write(m)

def jabberlog(bot, ievent):
    if ievent.botoutput:
        chan = ievent.to
    else:
        chan = ievent.channel
    m = {
        'timestamp': timestr(),
        'separator': format_opt('separator'),
        'event_prefix': format_opt('event_prefix'),
        'network': bot.networkname,
        'nick': ievent.nick,
        'target': chan,
    }
    if ievent.cmnd == 'Message':
            m.update({
                'type': 'comment',
                'txt': ievent.txt.strip(),
            })
    elif ievent.cmnd == 'Presence':
            if ievent.type == 'unavailable':
                m.update({
                    'type': 'part',
                    'txt': "%s left"%ievent.nick
                })
            else:
                m.update({
                    'type': 'join',
                    'txt': "%s joined"%ievent.nick
                })
    if m['txt']:
        write(m)

def prechatlogcb(bot, ievent):
    """Check if event should be logged.  QUIT and NICK are not channel
    specific, so we will check each channel in log()."""
    if not ievent.msg and [bot.name, ievent.channel] in \
            cfg.get('channels'):
        return 1
    if ievent.cmnd in ('QUIT', 'NICK'):
        return 1
    if ievent.cmnd == 'NOTICE':
        if [bot.name, ievent.arguments[0]] in cfg.get('channels'):
            return 1

def chatlogcb(bot, ievent):
    log(bot, ievent)

def jabberprechatlogcb(bot, ievent):
    if not ievent.groupchat:
        return 0
    if [bot.name, ievent.channel] in cfg.get('channels'):
        return 1

def jabberchatlogcb(bot, ievent):
    jabberlog(bot, ievent)

def handle_chatlogon(bot, ievent):
    chan = ievent.channel
    if [bot.name, chan] not in cfg.get('channels'):
        cfg['channels'].append([bot.name, chan])
        cfg.save()
        ievent.reply('chatlog enabled on (%s,%s)' % (bot.name, chan))
    else:
        ievent.reply('chatlog already enabled on (%s,%s)' % (bot.name, chan))

cmnds.add('chatlog-on', handle_chatlogon, 'OPER')
examples.add('chatlog-on', 'enable chatlog on <channel> or the channel \
the commands is given in', '1) chatlog-on 2) chatlog-on #dunkbots')

def handle_chatlogoff(bot, ievent):
    try:
        cfg['channels'].remove([bot.name, ievent.channel])
        cfg.save()
    except ValueError:
        ievent.reply('chatlog is not enabled in (%s,%s)' % (bot.name, \
ievent.channel))
        return
    ievent.reply('chatlog disabled on (%s,%s)' % (bot.name, ievent.channel))

cmnds.add('chatlog-off', handle_chatlogoff, 'OPER')
examples.add('chatlog-off', 'disable chatlog on <channel> or the channel \
the commands is given in', '1) chatlog-off 2) chatlog-off #dunkbots')

plughelp.add('chatlog', """
chatlog can log chats to mutliple backends.  Currently, only the file and db 
backends are supported.  

If you are using the file backend, there are two 
related configuration points; format and basepath.  Format dictates how the
logs are output.  There are currently two supported formats; simple and supy.
supy emulates the output logs of the excellent supybot ircbot.  Simple, as 
the name applies is some what simpler, although it logs comments and all
other lines types into seperate files.  Please create a ticket if you would
like another format supported.   

If you are using the db backend, then you must specify the database 
parameters.  This follows the same convention as gozerbot itself.  The db-
prefixed parameters are the ones you are after.  The chatlog plugin does
not create an table for chatlogs, this is up to the owner.  The owner will
need to create a table named chatlog with (time, network, target, nick, 
type, message) fields.  If there is an id field, it should be auto 
incrementing.  It is suggested to put and index on (network, target). 
target is usually a channel name.  

nologprefix is used to allow private messages that won't be logged by the bot.
The will be replaced by nologmsg.  

If you are looking for a great way to display your logs on the web, consider 
the irclogs trac plugin.  
</shameless-plug>
""")



# plugs/8b.py
#
#
#

""" eight ball """

__copyright__ = 'this file is in the public domain'
__revision__ = '$Id: m8b.py 1120 2006-12-30 12:00:00Z deck $'

from gozerbot.generic import handle_exception
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
from gozerbot.tests import tests
import re, random

plughelp.add('8b', 'Ask the magic 8 ball')

balltxt=[
    "Signs point to yes.",
    "Yes.",
    "Most likely.",
    "Without a doubt.",
    "Yes - definitely.",
    "As I see it, yes.",
    "You may rely on it.",
    "Outlook good.",
    "It is certain.",
    "It is decidedly so.",
    "Reply hazy, try again.",
    "Better not tell you now.",
    "Ask again later.",
    "Concentrate and ask again.",
    "Cannot predict now.",
    "My sources say no.",
    "Very doubtful.",
    "My reply is no.",
    "Outlook not so good.",
    "Don't count on it."
    ]

def handle_8b(bot, ievent):
    ievent.reply(random.choice(balltxt))

cmnds.add('8b', handle_8b, 'USER')
examples.add('8b', 'show what the magic 8 ball has to say', '8b')
tests.add('8b')



# gozerplugs/anon.py
#
#

__license__ = 'this file is in the public domain'
__gendocfirst__ = ['anon-enable', ]
__depend__ = ['throttle', ]

# gozerbot imports
from gozerbot.generic import rlog, jsonstring
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.callbacks import callbacks
from gozerbot.users import users
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.plughelp import plughelp
from gozerbot.tests import tests
from gozerplugs.throttle import state as throttlestate

plughelp.add('anon', 'allow registering of anon users on JOIN')

# plugin config
cfg = PersistConfig()
cfg.define('enable', []) # (bot.name, ievent.channel) pair
cfg.define('perms', ['USER', ])

def anonpre(bot, ievent):

    """ anon callback precondition """

    if 'OPER' not in cfg.get('perms') and \
jsonstring([bot.name, ievent.channel]) in cfg.get('enable'):
        return 1

def anoncb(bot, ievent):

    """ anon callback """

    try:
        username = users.getname(ievent.userhost)
        if not username:

            if users.add(ievent.nick, [ievent.userhost, ], perms = cfg.get('perms')):
                throttlestate['level'][ievent.userhost] = 10
                throttlestate.save()
                rlog(100, 'register', 'added %s (%s)' % (ievent.nick, ievent.userhost))

                if bot.cfg['type'] == 'irc':
                    bot.say(ievent.nick, "you have been added to the bots \
user database .. see %shelp for help" % bot.channels[ievent.channel]['cc'])
                else:
                    bot.say(ievent.jid, "you have been added to the bots \
user database .. see %shelp for help" % bot.channels[ievent.channel]['cc'])

            else:
                rlog(100, 'register' , "username %s already exists .. can't add %s" % (ievent.nick, ievent.userhost))

    except Exception, ex:
        rlog(100, 'register', 'failed to add %s (%s) .. reason: %s' % (ievent.nick, ievent.userhost, str(ex)))

callbacks.add('JOIN', anoncb, anonpre, threaded=True)
callbacks.add('Presence', anoncb, anonpre, threaded=True)
tests.add('anon-enable --chan #dunkbots').fakein(':dunker!mekker@127.0.0.1 JOIN #dunkbots').sleep(3).add('delete dunker').add('anon-disable --chan #dunkbots')

def handle_anonenable(bot, ievent):

    """ enable anon plugin .. register new users automatically. """

    cfg.append('enable', jsonstring([bot.name, ievent.channel]))
    ievent.reply('anon enabled on (%s,%s)' % (bot.name, ievent.channel))

cmnds.add('anon-enable', handle_anonenable, 'OPER')
examples.add('anon-enable', 'enable anon register', 'anon-enable')
tests.add('anon-enable', 'anon enabled')

def handle_anondisable(bot, ievent):

    """ disable anon registration. """

    try:
        cfg.remove('enable', jsonstring([bot.name, ievent.channel]))
        ievent.reply('anon disabled on (%s,%s)' % (bot.name, ievent.channel))
    except (KeyError, ValueError):
        ievent.reply('anon is not enabled on (%s,%s)' % (bot.name, \
ievent.channel))

cmnds.add('anon-disable', handle_anondisable, 'OPER')
examples.add('anon-disable', 'disable anon register', 'anon-disable')
tests.add('anon-disable', 'anon disabled')



# plugs/greeting.py
#
#

""" say greet message when user joins """

__copyright__ = 'this file is in the public domain'

from gozerbot.utils.generic import convertpickle
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.persist.pdol import Pdol
from gozerbot.datadir import datadir
from gozerbot.users import users
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.callbacks import callbacks
from gozerbot.plughelp import plughelp
import random, os

plughelp.add('greeting', 'the greeting plugin allows users to set messages \
to be said when they join a channel')

## UPGRADE PART

def upgrade():
    convertpickle(datadir + os.sep + 'old' + os.sep + 'greetings', \
datadir + os.sep + 'plugs' + os.sep + 'greeting' + os.sep + 'greetings')

cfg = PersistConfig()
cfg.define('enable', [])

greetings = None

def init():
    """ init the greeting plugin """
    global greetings
    greetings = Pdol(datadir + os.sep + 'plugs' + os.sep + 'greeting' + \
os.sep + 'greetings')
    if not greetings.data:
        upgrade()
        greetings = Pdol(datadir + os.sep + 'plugs' + os.sep + 'greeting' + \
os.sep + 'greetings')
    return 1

def greetingtest(bot, ievent):
    """ check if greeting callback should be called """
    if greetings and ievent.channel in cfg.get('enable'):
        return 1

def greetingcallback(bot, ievent):
    """ do the greeting """
    username = users.getname(ievent.userhost)
    try:
        greetingslist = greetings[username]
        if greetingslist:
            ievent.reply(random.choice(greetingslist))
    except KeyError:
        pass

callbacks.add('JOIN', greetingcallback, greetingtest)

def handle_greetingadd(bot, ievent):
    """ add greetings txt """
    if not greetings:
        ievent.reply('the greet plugin is not properly initialised')
        return
    if not ievent.rest:
        ievent.missing('<txt>')
        return
    username = users.getname(ievent.userhost)
    greetings.add(username, ievent.rest)
    greetings.save()
    ievent.reply('greeting message added')

cmnds.add('greeting-add', handle_greetingadd, 'USER')
examples.add('greeting-add', "add greeting message", 'greeting-add yooo dudes')

def handle_greetingdel(bot, ievent):
    """ delete greetings txt """
    if not greetings:
        ievent.reply('the greet plugin is not properly initialised')
        return
    try:
        nr = int(ievent.args[0])
    except (IndexError, ValueError):
        ievent.missing('<nr>')
        return
    username = users.getname(ievent.userhost)
    try:
        del greetings.data[username][nr]
    except (IndexError, KeyError):
        ievent.reply("can't delete greeting %s" % nr)
        return
    greetings.save()
    ievent.reply('greeting message %s removed' % nr)

cmnds.add('greeting-del', handle_greetingdel, 'USER')
examples.add('greeting-del', "delete greeting message", 'greeting-delete 1')

def handle_greetinglist(bot, ievent):
    """ list the greetings list of an user """
    if not greetings:
        ievent.reply('the greet plugin is not properly initialised')
        return
    username = users.getname(ievent.userhost)
    result = greetings.get(username)
    if result:
        ievent.reply("greetings: ", result, nr=0)
    else:
        ievent.reply('no greetings set')

cmnds.add('greeting-list', handle_greetinglist, 'USER')
examples.add('greeting-list', 'show greetings of user', 'greeting-list')



# plugs/url.py
#
#

from gozerbot.generic import handle_exception, rlog, convertpickle
from gozerbot.callbacks import callbacks
from gozerbot.commands import cmnds
from gozerbot.plughelp import plughelp
from gozerbot.examples import examples
from gozerbot.datadir import datadir
from gozerbot.persist.persiststate import PlugState
import re, os

plughelp.add('url', 'maintain searchable logs of urls')

def upgrade():
    convertpickle(datadir + os.sep + 'old' + os.sep + 'url', \
datadir + os.sep + 'plugs' + os.sep + 'url' + os.sep + 'state') 
    
re_url_match  = re.compile(u'((?:http|https)://\S+)')
state = None
initdone = False

def init():
    global state
    global initdone
    state = PlugState()
    state.define('urls', {})
    initdone = True
    return 1

def shutdown():
    if state:
        if len(state.data['urls']) > 0:
            state.save()
    return 1

def size():
    s = 0
    if not initdone:
        return s
    for i in state['urls'].values():
        for j in i.values():
            s += len(j)
    return s

def search(what, queue):
    global initdone
    rlog(10, 'url', 'searched for %s' % what)
    result = []

    if not initdone:
        return result

    try:
        for i in state['urls'].values():
            for urls in i.values():
                for url in urls:
                    if what in url:
                        result.append(url)
    except KeyError:
        pass
    for url in result:
        queue.put_nowait(url)

def urlpre(bot, ievent):
    return re_url_match.findall(ievent.txt)

def urlcb(bot, ievent):
    if not state:
        return 
    try:
        test_urls = re_url_match.findall(ievent.txt)
        for i in test_urls:
            if not state['urls'].has_key(bot.name):
                state['urls'][bot.name] = {}
            if not state['urls'][bot.name].has_key(ievent.channel):
                state['urls'][bot.name][ievent.channel] = []
            if not i in state['urls'][bot.name][ievent.channel]:
                state['urls'][bot.name][ievent.channel].append(i)  
    except Exception, ex:
        handle_exception()

callbacks.add('PRIVMSG', urlcb, urlpre, threaded=True)

def handle_urlsearch(bot, ievent):
    if not state:
        ievent.reply('rss state not initialized')
        return
    if not ievent.rest:
        ievent.missing('<what>')
        return
    result = []
    try:
        for i in state['urls'][bot.name][ievent.channel]:
            if ievent.rest in i:
                result.append(i)
    except KeyError:
        ievent.reply('no urls known for channel %s' % ievent.channel)
        return
    except Exception, ex:
        ievent.reply(str(ex))
        return
    if result:
        ievent.reply('results matching %s: ' % ievent.rest, result, nr=True)
    else:
        ievent.reply('no result found')
        return

cmnds.add('url-search', handle_urlsearch, ['USER', 'WEB', 'CLOUD'])
examples.add('url-search', 'search matching url entries', 'url-search gozerbot')

def handle_urlsearchall(bot, ievent):
    if not state:
        ievent.reply('rss state not initialized')
        return
    if not ievent.rest:
        ievent.missing('<what>')
        return
    result = []
    try:
        for i in state['urls'].values():
            for urls in i.values():
                for url in urls:
                    if ievent.rest in url:
                        result.append(url)
    except Exception, ex:
        ievent.reply(str(ex))
        return
    if result:
        ievent.reply('results matching %s: ' % ievent.rest, result, nr=True)
    else:
        ievent.reply('no result found')
        return

cmnds.add('url-searchall', handle_urlsearchall, ['USER', 'WEB', 'CLOUD'])
examples.add('url-searchall', 'search matching url entries', 'url-searchall \
gozerbot')

def handle_urlsize(bot, ievent):
    ievent.reply(str(size()))

cmnds.add('url-size', handle_urlsize, 'OPER')
examples.add('url-size', 'show number of urls in cache', 'url-size')
